import React, { useEffect, useState } from "react";
import { HiMiniStop, HiVolumeOff, HiVolumeUp } from "./icons.js";
export default function Speech({ text, pitch = 1, rate = 1, volume = 1, lang, voiceURI, startBtn = React.createElement(HiVolumeUp, null), pauseBtn = React.createElement(HiVolumeOff, null), stopBtn = React.createElement(HiMiniStop, null), useStopOverPause, onError = () => alert("Browser not supported! Try some other browser."), props = {}, children, }) {
    const [speechStatus, setSpeechStatus] = useState("stopped");
    const [useStop, setUseStop] = useState();
    const pause = () => { var _a; return speechStatus !== "paused" && ((_a = window.speechSynthesis) === null || _a === void 0 ? void 0 : _a.pause()); };
    const stop = () => { var _a; return speechStatus !== "stopped" && ((_a = window.speechSynthesis) === null || _a === void 0 ? void 0 : _a.cancel()); };
    function start() {
        const synth = window.speechSynthesis;
        if (!synth)
            return onError();
        setSpeechStatus("started");
        if (speechStatus === "paused")
            return synth.resume();
        if (synth.speaking)
            synth.cancel();
        const utterance = new window.SpeechSynthesisUtterance(text === null || text === void 0 ? void 0 : text.replace(/\s/g, " "));
        utterance.pitch = pitch;
        utterance.rate = rate;
        utterance.volume = volume;
        if (lang)
            utterance.lang = lang;
        if (voiceURI) {
            if (!Array.isArray(voiceURI))
                voiceURI = [voiceURI];
            const voices = synth.getVoices();
            for (let i = 0; i < voiceURI.length; i++) {
                const uri = voiceURI[i];
                const voice = voices.find((voice) => voice.voiceURI === uri);
                if (voice) {
                    utterance.voice = voice;
                    break;
                }
            }
        }
        function setStopped() {
            setSpeechStatus("stopped");
            utterance.onpause = null;
            utterance.onend = null;
            utterance.onerror = null;
            if (synth.paused)
                synth.cancel();
        }
        utterance.onpause = () => setSpeechStatus("paused");
        utterance.onend = setStopped;
        utterance.onerror = setStopped;
        synth.speak(utterance);
    }
    useEffect(() => {
        var _a, _b;
        setUseStop(useStopOverPause !== null && useStopOverPause !== void 0 ? useStopOverPause : (((_a = navigator.userAgentData) === null || _a === void 0 ? void 0 : _a.mobile) || false));
        (_b = window.speechSynthesis) === null || _b === void 0 ? void 0 : _b.cancel();
    }, []);
    return typeof children === "function" ? (children({ speechStatus, start, pause, stop })) : (React.createElement("div", Object.assign({ style: { display: "flex", columnGap: "1rem" } }, props),
        speechStatus !== "started" ? (React.createElement("span", { role: "button", onClick: start }, startBtn)) : useStop === false ? (React.createElement("span", { role: "button", onClick: pause }, pauseBtn)) : (React.createElement("span", { role: "button", onClick: stop }, stopBtn)),
        useStop === false && stopBtn && (React.createElement("span", { role: "button", onClick: stop }, stopBtn))));
}
